# Generated by roxygen2: do not edit by hand

export("experimentFiles<-")
export("otherData<-")
export("qdata<-")
export("sampleData<-")
export(MsExperiment)
export(MsExperimentFiles)
export(dbWriteSampleData)
export(existMsExperimentFiles)
export(experimentFiles)
export(linkSampleData)
export(otherData)
export(qdata)
export(readMsExperiment)
export(sampleData)
export(spectraSampleIndex)
exportClasses(MsExperiment)
exportClasses(MsExperimentFiles)
exportMethods("[")
exportMethods("spectra<-")
exportMethods(filterSpectra)
exportMethods(length)
exportMethods(show)
exportMethods(spectra)
importClassesFrom(IRanges,SimpleCharacterList)
importClassesFrom(QFeatures,QFeatures)
importClassesFrom(S4Vectors,Annotated)
importClassesFrom(S4Vectors,List)
importClassesFrom(S4Vectors,SimpleList)
importClassesFrom(Spectra,Spectra)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(DBI,dbDisconnect)
importFrom(DBI,dbExistsTable)
importFrom(DBI,dbGetQuery)
importFrom(DBI,dbListTables)
importFrom(DBI,dbRemoveTable)
importFrom(DBI,dbWriteTable)
importFrom(IRanges,CharacterList)
importFrom(IRanges,LogicalList)
importFrom(ProtGenerics,spectra)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,List)
importFrom(S4Vectors,findMatches)
importFrom(S4Vectors,from)
importFrom(S4Vectors,to)
importFrom(Spectra,msLevel)
importFrom(methods,"slot<-")
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(methods,validObject)
importMethodsFrom(BiocGenerics,dbconn)
importMethodsFrom(ProtGenerics,"spectra<-")
importMethodsFrom(ProtGenerics,filterSpectra)
importMethodsFrom(S4Vectors,"mcols<-")
importMethodsFrom(S4Vectors,"metadata<-")
importMethodsFrom(S4Vectors,mcols)
importMethodsFrom(S4Vectors,metadata)
importMethodsFrom(Spectra,Spectra)
importMethodsFrom(Spectra,peaksVariables)
importMethodsFrom(Spectra,selectSpectraVariables)
importMethodsFrom(Spectra,spectraVariables)
importMethodsFrom(methods,show)
